package Geo::SpatialDB::Path;
$Geo::SpatialDB::Path::VERSION = '0.000_001'; # TRIAL

$Geo::SpatialDB::Path::VERSION = '0.000001';use Moo 2;
use namespace::clean;

# ABSTRACT: A connected sequence of points on the map

has id  => ( is => 'rw' );
has seq => ( is => 'rw' );

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Geo::SpatialDB::Path - A connected sequence of points on the map

=head1 VERSION

version 0.000_001

=head1 AUTHOR

Michael Conrad <mike@nrdvana.net>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by Michael Conrad.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
