package Geo::SpatialDB::Storage;
$Geo::SpatialDB::Storage::VERSION = '0.000_001'; # TRIAL

$Geo::SpatialDB::Storage::VERSION = '0.000001';use Moo 2;
use namespace::clean;

# ABSTRACT: Base class for key/value storage appropriate for Geo::SpatialDB

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Geo::SpatialDB::Storage - Base class for key/value storage appropriate for Geo::SpatialDB

=head1 VERSION

version 0.000_001

=head1 AUTHOR

Michael Conrad <mike@nrdvana.net>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by Michael Conrad.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
