package Geo::SpatialDB::Location;
$Geo::SpatialDB::Location::VERSION = '0.000_001'; # TRIAL

$Geo::SpatialDB::Location::VERSION = '0.000001';use Moo 2;
use namespace::clean;

# ABSTRACT: A logical entity which can be described by a point on the map

extends 'Geo::SpatialDB::Entity';

has rel => ( is => 'rw' ); # arrayref of each entity ID related to this location
has lat => ( is => 'rw' );
has lon => ( is => 'rw' );
has rad => ( is => 'rw' ); # in meters

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Geo::SpatialDB::Location - A logical entity which can be described by a point on the map

=head1 VERSION

version 0.000_001

=head1 AUTHOR

Michael Conrad <mike@nrdvana.net>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by Michael Conrad.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
